/*
{
    "author": "",
    "color": "#fff4e7",
    "movement": true,
    "parameters": [
        {
            "default": 0.5,
            "name": "x"
        },
        {
            "default": 0.5,
            "name": "y"
        },
        {
            "default": 0.5,
            "name": "nee",
			"min": 0.2
        },
        {
            "default": 0.8,
            "name": "noo"
        },
        {
            "default": 0.5,
            "name": "goat"
        },
        {
            "default": 0.5,
            "name": "taint",
			"min": 0.2
        }
    ],
    "url": "",
    "uuid": "aafcbb45-cd26-4cbe-91cb-cb49bada1c16"
}
*/

#ifdef GL_ES
precision highp float;
#endif

const float sphereSize = 0.3;
const vec3 lightDir = vec3(-0.577, 0.577, 0.577);
const float PI = 3.14159265;
const float angle = 60.0;
const float fov = angle * 0.5 * PI / 180.0;

mat2 rot(float a) {
    float c = cos(a), s = sin(a);
    return mat2(c,s,-s,c);
}

vec3 trans(vec3 p) {
  return mod(p*nee, 4.0) - 2.0;
}

float distanceFunc(vec3 p){
    vec3 q = abs(trans(p));
    float hori = length(max(q - vec3(0.1, 2.0, 0.2), 0.0)) - 0.02;
    float vert = length(max(q - vec3(2.0, 0.1, 0.2), 0.0)) - 0.02;
    return min(hori, vert);
}

float distanceSphere(vec3 p, float r){
    return length(p) - r;
}

vec3 getNormal(vec3 p){
    float d = 0.0001;
    return normalize(vec3(
        distanceFunc(p + vec3(  d, 0.0, 0.0)) - distanceFunc(p + vec3( -d, 0.0, 0.0)),
        distanceFunc(p + vec3(0.0,   d, 0.0)) - distanceFunc(p + vec3(0.0,  -d, 0.0)),
        distanceFunc(p + vec3(0.0, 0.0,   d)) - distanceFunc(p + vec3(0.0, 0.0,  -d))
    ));
}

vec3 onRep(vec3 p, float interval) {
  return mod(p, interval) - interval * 0.5;
}

float distScene(vec3 p) {
  return distanceSphere(onRep(p, 4.0), 1.0);
}

float barDist( vec2 p, float b )
{
  return length(max(abs(p) - b, 0.0));
}

float sceneDist(vec3 p) {
  float bar_x = barDist(p.yz, 0.1);
  return bar_x;
}

void main(void){
    // fragment position
	float div = resolution.x/resolution.y;
	vec2 aspect = vec2(1,div);
	vec2 p = texCoord*aspect-vec2(x,y*div);
    //vec2 p = (gl_FragCoord.xy * 2.0 - resolution) / min(resolution.x, resolution.y);
    
    // camera
    vec3 cPos = vec3(0.0,  0.0,  -time*3.0*noo);
    vec3 cDir = vec3(0.0,  0.0, -1.0);
    vec3 cUp  = vec3(0.0,  1.0,  0.0);
    vec3 cSide = cross(cDir, cUp);
    float targetDepth = 1.0;
    
    // ray
    //vec3 ray = normalize(cSide * p.x + cUp * p.y + cDir * targetDepth);
    vec3 ray = normalize(vec3(sin(fov) * p.x, sin(fov*taint) * p.y, -cos(fov*exp(goat))));

    // marching loop
    float distance = 0.0;
    float rLen = 0.0;
    vec3  rPos = cPos;
    for(int i = 0; i < 64; i++){
        distance = distanceFunc(rPos)*2.0;
        rLen += distance;
        rPos = cPos + ray * rLen;
    }
    
    // hit check
    if(abs(distance) < 0.001){
        vec3 normal = getNormal(rPos);
        float diff = clamp(dot(lightDir, normal), 0.1, 1.0);
        gl_FragColor = vec4(vec3(diff*color.rgb*opacity), alpha);
    }else{
        gl_FragColor = vec4(vec3(0.0), 1.0);
    }
}
